
#include "fivewin.ch"

//-------------------------------------------------------------------------//

CLASS TAnimaBmp FROM TControl

      DATA cBitmap
      DATA oBmp
      DATA nX, nY
      DATA hBitmap, hPalette
      DATA oTimer
      DATA nBmps, nSpeed, nActual
      DATA nHBmp, nWBmp, nWPage

      DATA lStop, lWorking, lPainted, lFromTimer

      CLASSDATA lRegistered AS LOGICAL

      METHOD New( nTop, nLeft, nWidth, nHeight, cBitmap, nBmps,;
                  nSpeed, oWnd ) CONSTRUCTOR

      METHOD Redefine( oWnd, nId, cBitmap, nBmps, nSpeed ) CONSTRUCTOR

      METHOD Go()

      METHOD Stop( nGlip )  

      METHOD Destroy() 

      METHOD Initiate( hDlg ) INLINE Super:Initiate( hDlg ), ::Default()

      METHOD Display() INLINE ::BeginPaint(), ::Paint(), ::EndPaint()

      METHOD Paint()

      METHOD LoadBmp( cBitmap )

      METHOD End() INLINE ::Destroy()
   
      METHOD FreeBitmap()

      METHOD Default()

ENDCLASS

//-------------------------------------------------------------------------//

METHOD New( nTop, nLeft, nWidth, nHeight, cBitmap, nBmps, nSpeed, oWnd ) CLASS TAnimaBmp

   DEFAULT nBmps:= 0, nSpeed:= 4      // decimos de segundo

   DEFAULT nTop   := 0, nLeft := 0, oWnd := GetWndDefault()
   DEFAULT nWidth := 100, nHeight := 100

   ::nStyle     := nOR( WS_CHILD, WS_VISIBLE )
   ::nId        := ::GetNewId()
   ::oWnd       := oWnd
   ::cBitmap    := cBitmap
   ::hBitmap    := 0
   ::hPalette   := 0
   ::nBmps      := nBmps
   ::nSpeed     := nSpeed
   ::lStop      := .f.
   ::lWorking   := .f.
   ::lPainted   := .f.
   ::lFromTimer := .f.
   ::nActual    := 0
   ::nX         := 0
   ::nY         := 0

   ::nTop       := nTop
   ::nLeft      := nLeft
   ::nBottom    := nTop + nHeight
   ::nRight     := nLeft + nWidth

   ::Register( nOR( CS_VREDRAW, CS_HREDRAW ) )

   if ::oWnd != Nil
      ::SetColor( ::oWnd:nClrText, ::oWnd:nClrPane )
   endif
 
   IF ! Empty( oWnd:hWnd )
      ::Create()
      ::Default()
      ::lVisible = .t.
      oWnd:AddControl( Self )
   ELSE
      oWnd:DefControl( Self )
      ::lVisible  = .f.
   ENDIF

RETURN Self

//-------------------------------------------------------------------------//

METHOD Redefine( oWnd, nId, cBitmap, nBmps, nSpeed ) CLASS TAnimaBmp

   DEFAULT nBmps:= 0    ,;
           nSpeed := 4  ,;             // decimos de segundo
           oWnd := GetWndDefault()     

   ::oWnd       := oWnd
   ::nId        := nId
   ::cBitmap    := cBitmap
   ::hBitmap    := 0
   ::hPalette   := 0
   ::nBmps      := nBmps
   ::nSpeed     := nSpeed
   ::lStop      := .f.
   ::lWorking   := .f.
   ::lPainted   := .f.
   ::lFromTimer := .f.
   ::nActual    := 0
   ::nX         := 0
   ::nY         := 0

   ::Register( nOR( CS_VREDRAW, CS_HREDRAW ) )

   if oWnd != nil
      ::SetColor( ::oWnd:nClrText, ::oWnd:nClrPane )
      oWnd:DefControl( Self )
   ENDIF

RETURN Self

//-------------------------------------------------------------------------//

METHOD Default() CLASS TAnimaBmp

   ::LoadBmp( ::cBitmap )

   ::SetSize( ::nWPage, ::nHBmp, .t. )

   if ::oWnd != Nil
      ::SetColor( ::oWnd:nClrText, ::oWnd:nClrPane )
   endif

   if ::hBitmap != 0
      PalBmpNew( ::hWnd, ::hBitmap, ::hPalette )
   endif

   DEFINE TIMER ::oTimer ;
          INTERVAL ::nSpeed * 100  ;
          ACTION ( ::lFromTimer := .t.,::Refresh() ) OF Self

RETURN Nil

//-------------------------------------------------------------------------//

METHOD Go()  CLASS TAnimaBmp

   ::lStop:= .f.  
   ::lPainted:= .f.
   ::lFromTimer:= .t.
   ::Refresh()

RETURN Nil

//-------------------------------------------------------------------------//

METHOD Stop( nGlip )  CLASS TAnimaBmp

   if ::lStop
      RETURN Nil
   endif

   if ::oTimer:lActive
      ::oTimer:DeActivate()
   endif

   if nGlip != Nil
      ::nActual := Max( nGlip, 0 )
      ::nActual := Min( nGlip, ::nBmps )
   endif
   ::lFromTimer := .t.
   ::Refresh()
   ::lStop:= .t.

RETURN Nil

//-------------------------------------------------------------------------//

METHOD Paint() CLASS TAnimaBmp

   // tone( 1000, 1 )

   IF !::lPainted .and. !::lStop
      ::nActual  := 0
      ::lPainted := .t.

      ::GetDC()
      ::nY := 0
      PALBMPDraw( ::hDC, ::nX, ::nY, ::hBitmap, ::hPalette,;
                  ::nWBmp, ::nHBmp,, .t., ::oWnd:nClrPane )
      ::ReleaseDC()

      ACTIVATE TIMER ::oTimer
      RETURN Nil

   ENDIF

   ::GetDC()

   IF ::lFromTimer .and. !::lWorking

      ::lWorking := .t.

      if !::lStop 
         ::nActual++
      endif

      ::nActual:= Max( ::nActual, 1 )

      IF ::nActual > ::nBmps
         ::nActual := 1
      ENDIF

      ::nY := ( ( ::nActual * ::nWPage ) - ::nWPage ) * (-1)

   ENDIF

   PALBMPDraw( ::hDC, ::nX, ::nY, ::hBitmap, ::hPalette,;
                      ::nWBmp, ::nHBmp,, .t., ::oWnd:nClrPane ) 

   ::ReleaseDC()

   ::lFromTimer := .f.
   ::lWorking   := .f.
   // sysrefresh()

   IF ::bPainted != nil
      eval( ::bPainted, self )
   ENDIF

RETURN Nil

//-------------------------------------------------------------------------//

METHOD LoadBmp( cBitmap ) CLASS TAnimaBmp

   local aBmpPal

   ::cBitmap := cBitmap

   ::GetDC()

   IF "." $ ::cBitmap
      IF File( ::cBitmap )
         aBmpPal := PalBmpRead( ::hDC, ::cBitmap )
         ::hBitmap := aBmpPal[ 1 ]
         ::hPalette:= aBmpPal[ 2 ]
      ENDIF

   ELSE

      aBmpPal := PalBmpLoad( cBitmap )
      ::hBitmap  := aBmpPal[ 1 ]
      ::hPalette := aBmpPal[ 2 ]

   ENDIF

   ::nHBmp := nBmpHeight( ::hBitmap )
   ::nWBmp := nBmpWidth( ::hBitmap )

   IF ::nBmps = 0
      ::nBmps := ::nWBmp / ::nHBmp
   ENDIF
   ::nWPage:= ::nWBmp / ::nBmps

   ::ReleaseDC()

RETURN Nil

//-------------------------------------------------------------------------//

METHOD FreeBitmap() CLASS TAnimaBmp

   IF ::hBitmap != 0 .or. ::hPalette != 0
      PalBmpFree( ::hBitmap, ::hPalette )
   ENDIF

   ::hBitmap  = 0
   ::hPalette = 0

RETURN Nil

//-------------------------------------------------------------------------//

METHOD Destroy() CLASS TAnimaBmp

   LOCAL I

   ::FreeBitmap()

   if ::hBitmap != 0
      LogFile( "resources.txt", { "Destroy BMP failed from TAnimaBmp", ::hBitmap } )

      For I=1 to 10
          LogFile( "resources.txt", { Procname(i) } )
      Next I

   endif

   ::oTimer:End()

   if ::hWnd != 0
      Super:Destroy()
   endif

return nil

//-------------------------------------------------------------------------//

